@@ -61,9 +61,12 @@ def check_mail_config(job):
 
     defaul_port=dict(normal=25, ssl=465, tls=587)
     
-    mail=boolean.get(job.get('mail', 'yes'), None)
+    mail=job.get('mail', 'yes')
+    if mail!='fail':
+        mail=boolean.get(mail, None)
+    
     if mail==None:
-        errors['mail']='boolean must have value in (on, yes, true, 1, off, no, false and 0)'
+        errors['mail']='mail must have a value in (yes, no, fail)'
     else:
         job['mail']=mail
 
@@ -292,9 +295,10 @@ def quoted_string_list(st):
     return lst
 
 # ---------------------------------------------------------------------------
-def sendmail(job, subject, text, attachments, log):
+def sendmail(job, subject, text, attachments, log, status):
 
-    if not job['mail'] and not job['magikmon']:
+    errmsg=''     
+    if (job['mail']==False or (job['mail']=='fail' and status=='OK')) and not job['magikmon']:
         return None
 
     sender=job['sender']
@@ -310,6 +314,7 @@ def sendmail(job, subject, text, attachments, log):
     msg['To'] =  ', '.join([ formataddr((recipient, recipient)) for recipient in recipients ])
     msg['Date'] = formatdate(localtime=True)
     msg['Subject'] = subject
+    msg['Messsage-Id']=email.Utils.make_msgid('mksbackup')
 
     core=MIMEText(text.encode('utf-8'), 'plain', 'utf-8')
     msg.attach(core)
@@ -386,8 +391,7 @@ def sendmail(job, subject, text, attachments, log):
                         log.info('"%s" removed from the list of recipients', recipient)
                         recipients.remove(recipient)
                 
-    errmsg=''     
-    if job['mail'] and recipients:
+    if (job['mail']==True or (job['mail']=='fail' and status!='OK')) and recipients:
         smtp_host=job['smtp_host']
         smtp_port=job['smtp_port']
         smtp_mode=job['smtp_mode']
@@ -444,10 +448,24 @@ def sendmail(job, subject, text, attachments, log):
 
     return errmsg
 
+# ---------------------------------------------------------------------------
+def write_status(status, directory, job_name, log=None):
+    
+    filename=os.path.join(directory, job_name+'.txt')
+    try:
+        f=open(filename, 'w')
+        f.write(status.encode('utf-8'))
+        f.close()
+    except Exception, e:
+        if log:
+            log.warning('cannot write status into file: "%s"', filename)
+    else:
+        if log:
+            log.info('status written into file: "%s"', filename)
 
 # ---------------------------------------------------------------------------
 def send_mail_report(job, result, manager):
-    if not job['mail'] and not job['magikmon']:
+    if (job['mail']==False or (job['mail']=='fail' and status=='OK')) and not job['magikmon']:
         return None
     
     backup_status, status, attachments=result
@@ -464,19 +482,19 @@ def send_mail_report(job, result, manager):
         if line:
            manager.log.info('    %s', line)
     
-    return sendmail(job, subject, msg_body, attachments, manager.log)
+    return sendmail(job, subject, msg_body, attachments, manager.log, backup_status)
 
 
 # ---------------------------------------------------------------------------
-def param_encode(lst, encoding):
+def param_encode(lst, encoding, errors='strict'):
     """return a new encoded list of string"""
     if isinstance(lst, list):
         newlst=lst[:]
         for i, v in enumerate(lst):
-            newlst[i]=v.encode(encoding)
+            newlst[i]=v.encode(encoding, errors)
         return newlst
     elif isinstance(lst, unicode):
-        return lst.encode(encoding)
+        return lst.encode(encoding, errors)
     else:
         return lst
 
@@ -560,7 +578,7 @@ class Destinations:
         globals['nmonthname']=night_day.strftime('%b')
         
         epoch=calendar.timegm(backup_day.timetuple())
-        globals['epoch']=str(int(epoch))
+        globals['epoch']=int(epoch)
         globals['week']='0'
         globals['nweek']='0'
         globals['nmonthweek']=night_day.day/7
@@ -595,11 +613,12 @@ class Destinations:
 
             #print 'MATCH', typ, target
             rawtarget=target
-            for exp in re.findall('\{([^}]*)\}', target):
+            for exp in re.findall('\$\{([^}]*)\}', target):
+                # print 'exp=', exp
                 try:
                     target=target.replace('${%s}' % (exp,), str(eval(exp, globals, {})))
                 except Exception, e:
-                    raise DestinationSyntaxError, 'Invalid expression: "%s"' % (exp, )
+                    raise DestinationSyntaxError, 'Invalid expression: "%s" (%s)' % (exp, e)
             target=target.encode('utf-8')
             try:
                 target=backup_day.strftime(target)
@@ -721,4 +740,4 @@ class UnixRegistry:
         filedata.close()
         
     def Close(self):
-        pass
\ No newline at end of file
+        pass
