@@ -4,7 +4,7 @@
 # mksbackup ghettovcb frontent
 #
 
-import os, string, subprocess, fnmatch, platform, time
+import os, string, subprocess, fnmatch, platform, time, StringIO
 import posixpath, ftplib
 import threading, random
 
@@ -19,7 +19,7 @@ from archiver import *
 import ftpd
 import cron
 
-GHETTO_VERSION=[ '2011_03_14_1', '2011_06_28_1' ] 
+GHETTO_VERSION=[ '2011_03_14_1', '2011_06_28_1', '2011_11_19_1', ] 
 
 exit_code_re=re.compile('^\s*exit_code\s*=\s*([0-9]+)\s*$')
 
@@ -69,7 +69,7 @@ class MySCPClient(scp.SCPClient):
             self.channel.send('\x01')
             raise scp.SCPException('Bad directory format')
         else:
-            print 'ASX', self._recv_dir, self._recv_topdir
+            # print 'ASX', self._recv_dir, self._recv_topdir
             if self._recv_dir==self._recv_topdir and (not os.path.exists(self._recv_dir) or os.path.isdir(self._recv_dir)):
                 # use self._recv_dir instead of os.path.join(self._recv_dir, parts[2])
                 path=self._recv_dir
@@ -155,6 +155,8 @@ class Ghettovcb(Archiver):
         self.host=None
         self.login=None
         self.password=None
+        self.identity_file=None
+        self.auth_key=None
         self.local=None
         self.remote_temp=None
         self.vm_list=[]
@@ -233,15 +235,40 @@ class Ghettovcb(Archiver):
             if not (0<self.port and self.port<65535):
                 errors['port']='must be an integer between 1 and 65535'
 
-        for name in ('login', 'password', 'local'):
+        for name in ('login', 'local'):
             value=job.get(name, None)
             if not value:
                 errors[name]='option mandatory'
             else:
                 setattr(self, name, value)
+
+        for name in ('identity_file', 'password'):
+            value=job.get(name, None)
+            if value:
+                setattr(self, name, value)
                 if name=='password':
                     self.register_password(value)
 
+        if self.identity_file==None and self.password==None:
+                errors['password']=errors['identity_file']='one of "password or identity_file or both must be specified'
+        elif self.identity_file:
+            try:
+                self.auth_key=open(self.identity_file).read()
+            except Exception:
+                errors['identity_file']='cannot open identity file: "%s"' % (self.identity_file, )
+
+            if 'identity_file' not in errors:
+                keyfile=StringIO.StringIO(self.auth_key)
+                try:
+                    # self.password is valid or None
+                    self.auth_key=paramiko.RSAKey.from_private_key(keyfile, self.password)
+                except Exception, e:
+                    keyfile=StringIO.StringIO(self.auth_key)
+                    try:
+                        self.auth_key=paramiko.DSSKey.from_private_key(keyfile, self.password)
+                    except Exception, e:
+                        errors['identity_file']='invalid key or bad password: %s' % (e, )
+
         self.remote_temp=job.get('remote_temp', '/tmp')
 
         try:
@@ -308,10 +335,10 @@ class Ghettovcb(Archiver):
                 # check ghettoVCB version
                 self.ghettovcb_version=self.default_var.get('LAST_MODIFIED_DATE','')+'_'+(self.default_var.get('VER','') or self.default_var.get('VERSION', ''))
                 self.ghettovcb_wait_nfs=self.ghetto_version_format1.match(self.default_var.get('LAST_MODIFIED_DATE',''))!=None
-                if self.ghettovcb_wait_nfs:
+                if self.ghettovcb_wait_nfs or 'mksbackup' in self.ghettovcb_version:
                     log.debug('this version of ghettovcb wait before to unmount the NFS share')
                 
-                if not self.ghettovcb_version in GHETTO_VERSION:
+                if not self.ghettovcb_version in GHETTO_VERSION and not 'mksbackup' in self.ghettovcb_version:
                     warnings['local']='This version of ghettoVCB.sh (%s) script has not been tested with MKSBackup, be warned ! Check the download page for more.' % (self.ghettovcb_version, )
                 
             # read and extract global_conf
@@ -355,10 +382,26 @@ class Ghettovcb(Archiver):
                     
         if not errors:
             # upload and setup files on the VMWARE server
-
-            t=paramiko.Transport((self.host, self.port))
-            t.connect(username=self.login, password=self.password)
+            try:
+                t=paramiko.Transport((self.host, self.port))
+                t.start_client()
+            except (socket.error, paramiko.SSHException), e:
+                errors['host']=errors['port']=str(e)
             
+        if not errors:
+            try:
+                if self.auth_key:
+                    t.auth_publickey(self.login, self.auth_key)
+                else:
+                    t.auth_password(self.login, self.password)
+            except paramiko.AuthenticationException, e:
+                t.close()
+                if self.auth_key:
+                    errors['identity_file']=str(e)
+                else:
+                    errors['password']=str(e)
+                    
+        if not errors:
             scpclient=MySCPClient(t)
             scpclient.put(self.local, self.remote_temp, recursive=True)
 
@@ -718,13 +761,15 @@ class Ghettovcb(Archiver):
                 for line in output.split('\n'):
                     if not line:
                         continue
-                    block_number, size, c_epoch, m_epoch, filename=line.split(':', 5)
+                    block_number, size, c_epoch, m_epoch, filename=line.split(':', 4)
                     if filename.startswith('.lck-'):
                         continue
                     
-                    files[filename]=(max(int(c_epoch), int(m_epoch)), line)
-                         
-                for filename, (epoch, line) in files.iteritems():
+                    files[filename]=(max(int(c_epoch), int(m_epoch)), filename, line)
+                    
+                sorted_files=files.values()
+                sorted_files.sort(lambda x, y:x[0]-y[0])
+                for epoch, filename, line in sorted_files:
                     if line in self.uploaded_filenames:
                         continue
                     if not finished:
@@ -758,8 +803,7 @@ class Ghettovcb(Archiver):
                                 ftp_pre_create_dir(ftp['host'], ftp['port'], ftp['username'], ftp['password'], dest_dir)
                             except Exception, e:
                                 self.log.error('cannot create directory, try to create it yourself on ftp server: %s', dest_dir)
-                            else:
-                                self.uploaded_filenames[dest_dir]=None
+                            self.uploaded_filenames[dest_dir]=None
                             
                         ftp_line='ftpput -P %s -u "%s" -p "%s" %s "%s" "%s"' % (con['port'], con['username'], con['password'], con['host'], destination, source)
                         self.download_script+='echo -n download "%s"\n' % (destination, )
@@ -771,6 +815,7 @@ class Ghettovcb(Archiver):
                         if exit_code==0:
                             self.log.debug('file uploaded: %s', destination)
                             self.uploaded_filenames[line]=None
+                            wait=1 # try next quickly
                         else:
                             if finished:
                                 status='ERR'
@@ -806,6 +851,9 @@ class Ghettovcb(Archiver):
                                     self.log.debug('epoch=%d file=%s', esx_epoch, line)
                                 else:
                                     self.log.info('download failed, retry later %s (%s)', destination, e)
+                            else:
+                                wait=1 # try next quickly
+                                self.uploaded_filenames[line]=None
                         
                         else:
                             # use local SCP
@@ -829,12 +877,12 @@ class Ghettovcb(Archiver):
                                     self.log.debug('epoch=%d file=%s', esx_epoch, line)
                                 else:
                                     self.log.info('download failed, retry later %s (%s)', destination, scp_out+scp_err)
+                            else:
+                                self.uploaded_filenames[line]=None
                     
                     if not finished:
                         # reload the list to work on fresh data
-                        wait=1 
                         break
-                    
 
 
             end=time.time()                
@@ -868,8 +916,12 @@ class Ghettovcb(Archiver):
         start=int(time.time())
 
         self.t=t=paramiko.Transport((self.host, self.port))
+        t.start_client()
         # t.get_security_options()._set_ciphers(['blowfish-cbc', ]) # don't speed up anything and don't work with 3.5 
-        t.connect(username=self.login, password=self.password)
+        if self.auth_key:
+            t.auth_publickey(self.login, self.auth_key)
+        else:
+            t.auth_password(self.login, self.password)
         # t.set_keepalive(300) # make the backup fail
         chan=t.open_session()
         chan.set_combine_stderr(True)
