@@ -14,7 +14,7 @@ import xml.dom.minidom
 import win32wnet
 import win32netcon 
 
-from windows import WindowsErrorDecode, windows_list_dir
+from windows import WindowsErrorDecode, windows_list_dir, is_volumeid
 
 from archiver import *
 
@@ -26,25 +26,20 @@ def getvalue(parentnode, nodename):
 # ===========================================================================
 #
 # ===========================================================================
-class Wbadmin(Archiver):
+class WbadminBase(Archiver):
 
-    name='wbadmin'
-    exe='wbadmin.exe'
+    name='WBADMIN-BASE'
+    exe='WBADMIN-BASE'
     
-    types=dict(vsscopy='vsscopy', vssfull='vssfull', copy='copy') # copy is deprecated 
+    types=dict() 
     
     # -----------------------------------------------------------------------
     def __init__(self):
         Archiver.__init__(self)
-        
-        self.selection=None
-        self.verify=None
-        self.restricted=None
-        self.program_exe=None
+
+        self.destination=None
         self.type=None
         self.target=None
-        self.user=None
-        self.password=None
         self.options=[]
 
         systemroot=os.environ.get('SystemRoot', os.environ.get('windir', 'C:\\windows'))
@@ -69,7 +64,6 @@ class Wbadmin(Archiver):
         #
         # check job config
         #
-
         self.destination=job.get('destination', None)
         if self.destination:
             self.destination=self.destination.replace('\n','')
@@ -78,23 +72,6 @@ class Wbadmin(Archiver):
             except (DestinationSyntaxError, cron.CronException), e:
                 errors['destination']='syntax error: %s' % (str(e), )
 
-        self.include=job.get('include', None)
-        if self.include and not self.destination:
-            errors['include']='destination required'
-
-        self.user=job.get('user', None)
-        if self.user and not self.destination:
-            errors['user']='destination required'
-            
-        self.password=job.get('password', None)
-        self.register_password(self.password)
-            
-        if self.user and not self.destination:
-            errors['password']='destination required'
-
-        if (self.user and not self.password) or (not self.user and self.password):
-            errors['password']='login and password must be set together'
-
         options=job.get('options', '')
         try:
             options=quoted_string_list(options)
@@ -110,61 +87,8 @@ class Wbadmin(Archiver):
         extra+='wbadmin_bin=%s\n' % self.wbadmin_bin
         extra+='wevtutil_bin=%s\n' % self.wevtutil_bin
         
-        if not errors:
-            if not self.destination:
-                self.type, self.target='default', None
-            else:
-                self.type, self.target=self.destination.match(now, self.night_shift)
-    
-                if self.type!=None and self.type!='none':            
-                    # check the target
-                    if not os.path.isdir(self.target):
-                        try:
-                            os.makedirs(self.target)
-                        except WindowsError, e:
-                            unc=os.path.splitunc(self.target)[0]
-                            if unc and self.user and self.password:
-                                try:
-                                    win32wnet.WNetAddConnection2(win32netcon.RESOURCETYPE_DISK, None, param_encode(unc, manager.default_encoding), None, param_encode(self.user, manager.default_encoding), param_encode(self.password, manager.default_encoding))
-                                    try:
-                                        if not os.path.isdir(self.target):
-                                            try:
-                                                os.makedirs(self.target)
-                                            except WindowsError, e:
-                                                errors['destination']='directory not found: %s (%s)' % (self.target, WindowsErrorDecode(e))
-                                    finally:
-                                        win32wnet.WNetCancelConnection2(param_encode(unc, manager.default_encoding), 0, False)
-                                except Exception, e:
-                                    errors['destination']='directory not found: %s (%s)' % (self.target, WindowsErrorDecode(e))
-                            else:
-                                errors['destination']='directory not found: %s (%s)' % (self.target, WindowsErrorDecode(e))
-
-        if not errors:
-
-            if self.type!=None and self.type!='none':
-                
-                self.wba_args=[ self.exe, 'start', 'backup', '-quiet' ]
-                # TODO: remove this
-                
-                if self.target:
-                    self.wba_args.append('-backupTarget:%s' % ( self.target, ))
-                    
-                if self.include:
-                    self.wba_args.append('-include:%s' % ( self.include, ))
-                
-                if self.target.startswith('\\\\') and self.user and self.password:
-                    self.wba_args+=[ '-user:%s' % ( self.user, ), '-password:%s' % ( self.password,) ]
-                    
-                if self.type.lower()=='vssfull':
-                    self.wba_args.append('-vssFull')
-                elif self.type.lower()=='vsscopy':
-                    self.wba_args.append('-vssCopy')
-                    
-                self.wba_args.extend(self.options)
-                extra+='cmdline=%s\n' % self.hide_password('  '.join(self.wba_args))
-    
         return errors, warnings, extra
-
+    
     # -----------------------------------------------------------------------
     def run(self, command, job, manager):
 
@@ -194,6 +118,9 @@ class Wbadmin(Archiver):
         try:
             wba_process=subprocess.Popen(param_encode(self.wba_args, manager.default_encoding), executable=self.wbadmin_bin, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
             wba_out, wba_err=wba_process.communicate()
+        except UnicodeEncodeError, e:
+            st=self.hide_password(' '.join(param_encode(self.wba_args, manager.default_encoding, 'backslashreplace')))
+            raise RuntimeError, 'Unicode error, avoid the use of non us-ascii characters in your .ini file: '+st
         except Exception, e:
             log.error('Unexpected exception running command: %s', self.hide_password(' '.join(self.wba_args)))
             raise
@@ -277,7 +204,7 @@ class Wbadmin(Archiver):
         
         total_free_bytes, dir_out, dir_up_out, total_size=None, None, None, None
         
-        if self.target:
+        if self.target and not is_volumeid(self.target):
             status+='target=%s\r\n' % self.target
             
             if self.target[-1]==':':
@@ -390,19 +317,120 @@ class Wbadmin(Archiver):
             try:
                 value=open(backupfailurelog, 'r').read()
             except Exception, e:
-               log.error('reading backup failure log "%s": %s', backupfailurelog, WindowsErrorDecode(e, manager.default_encoding))
+               log.error('reading backup failure log "%s": %s', backupfailurelog.decode(manager.default_encoding, 'replace'), WindowsErrorDecode(e, manager.default_encoding))
             else:
                 if len(value)>2: # BOM
-                    status+='backup_failure_log=%s\r\n' % backupfailurelog
+                    status+=u'backup_failure_log=%s\r\n' % backupfailurelog.decode(manager.default_encoding, 'replace')
                     attachments.append(('backupfailurelog.txt', None, value, 'text', 'plain', 'utf-16'))
                
         return ev_status, status, attachments
 
+# ===========================================================================
+#
+# ===========================================================================
+class Wbadmin(WbadminBase):
+
+    name='wbadmin'
+    exe='wbadmin.exe'
+    
+    types=dict(vsscopy='vsscopy', vssfull='vssfull', copy='copy') # copy is deprecated 
+    
+    # -----------------------------------------------------------------------
+    def __init__(self):
+        WbadminBase.__init__(self)
+
+        self.user=None
+        self.password=None
+        
+    # -----------------------------------------------------------------------
+    def load(self, job, manager):
+        
+        errors, warnings, extra=WbadminBase.load(self, job, manager)
+
+        log=manager.log
+        now=manager.now
+
+        #
+        # check job config
+        #
+
+        self.include=job.get('include', None)
+        if self.include and not self.destination:
+            errors['include']='destination required'
+
+        self.user=job.get('user', None)
+        if self.user and not self.destination:
+            errors['user']='destination required'
+            
+        self.password=job.get('password', None)
+        self.register_password(self.password)
+            
+        if self.user and not self.destination:
+            errors['password']='destination required'
+
+        if (self.user and not self.password) or (not self.user and self.password):
+            errors['password']='login and password must be set together'
+
+        if not errors:
+            if not self.destination:
+                self.type, self.target='default', None
+            else:
+                self.type, self.target=self.destination.match(now, self.night_shift)
+                
+                if self.type!=None and self.type!='none':            
+                    # check the target
+                    if self.target and not is_volumeid(self.target) and not os.path.isdir(self.target):
+                        try:
+                            os.makedirs(self.target)
+                        except WindowsError, e:
+                            unc=os.path.splitunc(self.target)[0]
+                            if unc and self.user and self.password:
+                                try:
+                                    win32wnet.WNetAddConnection2(win32netcon.RESOURCETYPE_DISK, None, param_encode(unc, manager.default_encoding), None, param_encode(self.user, manager.default_encoding), param_encode(self.password, manager.default_encoding))
+                                    try:
+                                        if not os.path.isdir(self.target):
+                                            try:
+                                                os.makedirs(self.target)
+                                            except WindowsError, e:
+                                                errors['destination']='directory not found: %s (%s)' % (self.target, WindowsErrorDecode(e))
+                                    finally:
+                                        win32wnet.WNetCancelConnection2(param_encode(unc, manager.default_encoding), 0, False)
+                                except Exception, e:
+                                    errors['destination']='directory not found: %s (%s)' % (self.target, WindowsErrorDecode(e))
+                            else:
+                                errors['destination']='directory not found: %s (%s)' % (self.target, WindowsErrorDecode(e))
+
+        if not errors:
+
+            if self.type!=None and self.type!='none':
+                
+                self.wba_args=[ self.exe, 'start', 'backup', '-quiet' ]
+                # TODO: remove this
+                
+                if self.target:
+                    self.wba_args.append('-backupTarget:%s' % ( self.target, ))
+                    
+                if self.include:
+                    self.wba_args.append('-include:%s' % ( self.include, ))
+                
+                if self.target and self.target.startswith('\\\\') and not is_volumeid(self.target) and self.user and self.password:
+                    self.wba_args+=[ '-user:%s' % ( self.user, ), '-password:%s' % ( self.password,) ]
+                    
+                if self.type.lower()=='vssfull':
+                    self.wba_args.append('-vssFull')
+                elif self.type.lower()=='vsscopy':
+                    self.wba_args.append('-vssCopy')
+                    
+                self.wba_args.extend(self.options)
+                extra+='cmdline=%s\n' % self.hide_password('  '.join(self.wba_args))
+    
+        return errors, warnings, extra
+
 
 # ===========================================================================
 #
 # ===========================================================================
-class WbadminSys(Wbadmin):
+class WbadminSys(WbadminBase):
 
     name='wbadminsys'
     exe='wbadmin.exe'
@@ -415,22 +443,15 @@ class WbadminSys(Wbadmin):
         log=manager.log
         now=manager.now
 
-        errors, warnings, extra={}, {}, ''
+        errors, warnings, extra=WbadminBase.load(self, job, manager)
 
         #
         # check job config
         #
-
-        self.destination=job.get('destination', None)
+            
         if not self.destination:
             errors['destination']='destination required'
-        else:
-            self.destination=self.destination.replace('\n','')
-            try:
-                self.destination=Destinations(self.destination, self)
-            except (DestinationSyntaxError, cron.CronException), e:
-                errors['destination']='syntax error: %s' % (str(e), )
-            
+
         self.keepversions=job.get('keepversions', None)
         if self.keepversions:
             try:
@@ -447,8 +468,11 @@ class WbadminSys(Wbadmin):
     
             if self.type!=None and self.type!='none':
                 
-                self.wba_args=[ self.exe, 'start', 'systemstatebackup', '-quiet', '-backupTarget:%s' % ( self.target, ) ] 
-                
+                self.wba_args=[ self.exe, 'start', 'systemstatebackup', '-quiet']
+                if self.target:
+                    self.wba_args.append('-backupTarget:%s' % ( self.target, )) 
+
+                self.wba_args.extend(self.options)
                 extra+='cmdline=%s\n' % '  '.join(self.wba_args)
     
         return errors, warnings, extra
